@@ -1,9 +1,12 @@
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
-#include "texteditor/InputMethodPopup.h"
+//#include "texteditor/InputMethodPopup.h"
 
 #include <QMainWindow>
+#include "texteditor/AbstractCharacterBasedApplication.h"
+
+class TextEditorView;
 
 namespace Ui {
 class MainWindow;
@@ -30,6 +33,7 @@ private:
 
 	TextEditorEnginePtr engine();
 	Document *document();
+	TextEditorView *texteditor();
 protected:
 	void keyPressEvent(QKeyEvent *);
 
