@@ -2,7 +2,12 @@
 
 load '../../version.rb'
 
-$package = "guitar"
+$suffix = ENV['SUFFIX']
+if $suffix.nil?
+	$suffix = ''
+end
+
+$package = "guitar" + $suffix
 $arch = `./arch.rb`.strip
 $debname = "#{$package}_#{$version_a}.#{$version_b}.#{$version_c}_#{$arch}.deb"
 
