@@ -4,7 +4,12 @@ require 'fileutils'
 
 load '../../version.rb'
 
-$package = "guitar"
+$suffix = ENV['SUFFIX']
+if $suffix.nil?
+	$suffix = ''
+end
+
+$package = "guitar" + $suffix
 $maintainer = "nobody <nobody@example.com>"
 $version = "#{$version_a}.#{$version_b}.#{$version_c}"
 $workdir = "build"
@@ -39,7 +44,7 @@ Section: vcs
 Maintainer: #{$maintainer}
 Architecture: #{$arch}
 Version: #{$version}
-Depends: libqt5widgets5 (>= 5.5.0), libqt5xml5 (>= 5.5.0), libqt5svg5 (>= 5.5.0), zlib1g, git, file
+Depends: libqt5widgets5 (>= 5.11.0), libqt5xml5 (>= 5.11.0), libqt5svg5 (>= 5.11.0), zlib1g, git, libmagic1, libmagic-mgc
 Description: Git GUI Client
 ___
 }
