@@ -1,20 +1,22 @@
 
+#include "FileDiffWidget.h"
 #include "ApplicationGlobal.h"
 #include "BigDiffWindow.h"
-#include "FileDiffWidget.h"
 #include "GitDiff.h"
 #include "MainWindow.h"
 #include "Theme.h"
 #include "common/joinpath.h"
 #include "common/misc.h"
+#include "dtl/dtl.hpp"
 #include "ui_FileDiffWidget.h"
 #include <QBuffer>
 #include <QDebug>
+#include <QFile>
 #include <QKeyEvent>
 #include <QMenu>
 #include <QPainter>
 #include <QStyle>
-#include <QTextCodec>
+// #include <QTextCodec>
 #include <memory>
 
 enum {
@@ -22,7 +24,6 @@ enum {
 };
 
 struct FileDiffWidget::Private {
-	BasicMainWindow *mainwindow = nullptr;
 	FileDiffWidget::InitParam_ init_param_;
 	Git::CommitItemList commit_item_list;
 	std::vector<std::string> original_lines;
@@ -35,7 +36,9 @@ struct FileDiffWidget::Private {
 	int term_cursor_row = 0;
 	int term_cursor_col = 0;
 
-	QTextCodec *text_codec = nullptr;
+	std::shared_ptr<MyTextCodec> text_codec;
+
+	std::vector<std::pair<LineFragment, LineFragment>> linefragmentpair;
 };
 
 FileDiffWidget::FileDiffWidget(QWidget *parent)
@@ -52,9 +55,14 @@ FileDiffWidget::FileDiffWidget(QWidget *parent)
 		return makeDiffPixmap(pane, width, height);
 	}, global->theme);
 
+	// 左端のスライダー
 	connect(ui->widget_diff_slider, &FileDiffSliderWidget::valueChanged, this, &FileDiffWidget::scrollTo);
-	connect(ui->widget_diff_left->texteditor(), &TextEditorWidget::moved, this, &FileDiffWidget::onMoved);
-	connect(ui->widget_diff_right->texteditor(), &TextEditorWidget::moved, this, &FileDiffWidget::onMoved);
+
+	// 左のテキストビュー
+	connect(ui->widget_diff_left->texteditor(), &TextEditorView::moved, this, &FileDiffWidget::onMoved);
+
+	// 右のテキストビュー
+	connect(ui->widget_diff_right->texteditor(), &TextEditorView::moved, this, &FileDiffWidget::onMoved);
 
 	setFocusAcceptable(Qt::ClickFocus);
 	QWidget::setTabOrder(ui->widget_diff_slider, ui->widget_diff_left);
@@ -79,6 +87,24 @@ FileDiffWidget::~FileDiffWidget()
 	delete ui;
 }
 
+MainWindow *FileDiffWidget::mainwindow()
+{
+	return global->mainwindow;
+}
+
+/**
+ * @brief スクロールバーのセットアップ
+ * @param mw
+ */
+void FileDiffWidget::init()
+{
+	ui->widget_diff_left->bind(this, ui->verticalScrollBar, ui->horizontalScrollBar, mainwindow()->themeForTextEditor());
+	ui->widget_diff_right->bind(this, ui->verticalScrollBar, ui->horizontalScrollBar, mainwindow()->themeForTextEditor());
+
+	connect(ui->verticalScrollBar, &QAbstractSlider::valueChanged, this, &FileDiffWidget::onVerticalScrollValueChanged);
+	connect(ui->horizontalScrollBar, &QAbstractSlider::valueChanged, this, &FileDiffWidget::onHorizontalScrollValueChanged);
+}
+
 void FileDiffWidget::setViewType(FileViewType type)
 {
 	ui->widget_diff_left->setViewType(type);
@@ -96,36 +122,18 @@ FileDiffWidget::ViewStyle FileDiffWidget::viewstyle() const
 	return m->init_param_.view_style;
 }
 
-
-
-void FileDiffWidget::bind(BasicMainWindow *mw)
-{
-	Q_ASSERT(mw);
-	m->mainwindow = mw;
-	ui->widget_diff_left->bind(mw, this, ui->verticalScrollBar, ui->horizontalScrollBar, mw->themeForTextEditor());
-	ui->widget_diff_right->bind(mw, this, ui->verticalScrollBar, ui->horizontalScrollBar, mw->themeForTextEditor());
-
-	connect(ui->verticalScrollBar, &QAbstractSlider::valueChanged, this, &FileDiffWidget::onVerticalScrollValueChanged);
-	connect(ui->horizontalScrollBar, &QAbstractSlider::valueChanged, this, &FileDiffWidget::onHorizontalScrollValueChanged);
-}
-
-BasicMainWindow *FileDiffWidget::mainwindow()
-{
-	return m->mainwindow;
-}
-
-GitPtr FileDiffWidget::git()
+GitRunner FileDiffWidget::git()
 {
 	if (!mainwindow()) {
-		qDebug() << "Maybe, you forgot to call FileDiffWidget::bind() ?";
-		return GitPtr();
+		qDebug() << "Maybe, you forgot to call FileDiffWidget::bind()?";
+		return {};
 	}
 	return mainwindow()->git();
 }
 
-Git::Object FileDiffWidget::cat_file(GitPtr const &/*g*/, QString const &id)
+Git::Object FileDiffWidget::catFile(GitRunner g, QString const &id)
 {
-	return mainwindow()->cat_file(id);
+	return mainwindow()->catFile(g, id);
 }
 
 int FileDiffWidget::totalTextLines() const
@@ -135,6 +143,10 @@ int FileDiffWidget::totalTextLines() const
 
 void FileDiffWidget::clearDiffView()
 {
+	m->linefragmentpair.clear();
+
+	ui->widget_diff_left->clear();
+	ui->widget_diff_right->clear();
 	ui->widget_diff_slider->clear(false);
 }
 
@@ -187,7 +199,7 @@ void FileDiffWidget::makeSideBySideDiffData(Git::Diff const &diff, std::vector<s
 			auto ParseNumber = [&](){
 				size_t v = 0;
 				while (isdigit(*p & 0xff)) {
-					v = v * 10 + (*p - '0');
+					v = v * 10 + size_t(*p - '0');
 					p++;
 				}
 				return v;
@@ -216,7 +228,7 @@ void FileDiffWidget::makeSideBySideDiffData(Git::Diff const &diff, std::vector<s
 	size_t h = hunks.size();
 	while (linenum > 0 || h > 0) {
 		while (h > 0) {
-			int hunk_number = h - 1;
+			int hunk_number = int(h - 1);
 			HunkItem const &hi = hunks[hunk_number];
 			if (hi.pos + hi.len < linenum) {
 				break;
@@ -310,6 +322,7 @@ void FileDiffWidget::makeSideBySideDiffData(Git::Diff const &diff, std::vector<s
 
 void FileDiffWidget::setDiffText(Git::Diff const &diff, TextDiffLineList const &left, TextDiffLineList const &right)
 {
+	ASSERT_MAIN_THREAD();
 	m->max_line_length = 0;
 
 	enum Pane {
@@ -347,18 +360,60 @@ void FileDiffWidget::setDiffText(Git::Diff const &diff, TextDiffLineList const &
 	SetLineNumber(left, Pane::Left, &m->left_lines);
 	SetLineNumber(right, Pane::Right, &m->right_lines);
 
-	ui->widget_diff_left->setText(&m->left_lines, mainwindow(), diff.blob.a_id, diff.path);
-	ui->widget_diff_right->setText(&m->right_lines, mainwindow(), diff.blob.b_id, diff.path);
+	std::vector<LineFragment> left_dels;
+	std::vector<LineFragment> right_adds;
+
+	auto Do = [](TextDiffLineList const &lines, Document::Line::Type type, std::vector<LineFragment> *blocks){
+		int i = 0;
+		while (i < lines.size()) {
+			int n = 1;
+			while (i + n < lines.size()) {
+				if (lines[i + n].type != lines[i].type) break;
+				n++;
+			}
+			if (lines[i].type == type) {
+				blocks->emplace_back(type, i, n);
+			}
+			i += n;
+		}
+	};
+	Do(m->left_lines, Document::Line::Del, &left_dels);
+	Do(m->right_lines, Document::Line::Add, &right_adds);
+
+	m->linefragmentpair.clear();
+
+	std::size_t ileft = 0;
+	std::size_t iright = 0;
+	while (ileft < left_dels.size() && iright < right_adds.size()) {
+		if (left_dels[ileft].line_index < right_adds[iright].line_index) {
+			ileft++;
+		} else if (left_dels[ileft].line_index > right_adds[iright].line_index) {
+			iright++;
+		} else {
+			if (left_dels[ileft].line_count == right_adds[iright].line_count) {
+				m->linefragmentpair.emplace_back(left_dels[ileft], right_adds[iright]);
+			}
+			ileft++;
+			iright++;
+		}
+	}
+
+	ui->widget_diff_left->setText(&m->left_lines, diff.blob.a_id_or_path, diff.path);
+	ui->widget_diff_right->setText(&m->right_lines, diff.blob.b_id_or_path, diff.path);
 	refrectScrollBar();
 	ui->widget_diff_slider->clear(true);
 }
 
+/**
+ * @brief テキストか画像かでビューを切り替える
+ * @return
+ */
 FileViewType FileDiffWidget::setupPreviewWidget()
 {
 	clearDiffView();
 
-	QString mimetype_l = mainwindow()->determinFileType(m->init_param_.bytes_a, true);
-	QString mimetype_r = mainwindow()->determinFileType(m->init_param_.bytes_b, true);
+	std::string mimetype_l = mainwindow()->determineFileType(m->init_param_.bytes_a);
+	std::string mimetype_r = mainwindow()->determineFileType(m->init_param_.bytes_b);
 
 	if (misc::isImage(mimetype_l) || misc::isImage(mimetype_r)) { // image
 
@@ -366,8 +421,9 @@ FileViewType FileDiffWidget::setupPreviewWidget()
 		ui->horizontalScrollBar->setVisible(false);
 		ui->widget_diff_slider->setVisible(false);
 
-		ui->widget_diff_left->setImage(mimetype_l, m->init_param_.bytes_a, m->init_param_.diff.blob.a_id, m->init_param_.diff.path);
-		ui->widget_diff_right->setImage(mimetype_r, m->init_param_.bytes_b, m->init_param_.diff.blob.b_id, m->init_param_.diff.path);
+//		qDebug() << mimetype_l << m->init_param_.diff.blob.a_id_or_path << mimetype_r << m->init_param_.diff.blob.b_id_or_path;
+		ui->widget_diff_left->setImage(mimetype_l, m->init_param_.bytes_a, m->init_param_.diff.blob.a_id_or_path, m->init_param_.diff.path);
+		ui->widget_diff_right->setImage(mimetype_r, m->init_param_.bytes_b, m->init_param_.diff.blob.b_id_or_path, m->init_param_.diff.path);
 
 		return FileViewType::Image;
 
@@ -388,27 +444,33 @@ void FileDiffWidget::setSingleFile(QByteArray const &ba, QString const &id, QStr
 	m->init_param_.view_style = FileDiffWidget::ViewStyle::SingleFile;
 	m->init_param_.bytes_a = ba;
 	m->init_param_.diff.path = path;
-	m->init_param_.diff.blob.a_id = id;
+	m->init_param_.diff.blob.a_id_or_path = id;
 }
 
-void FileDiffWidget::setOriginalLines_(QByteArray const &ba)
+void FileDiffWidget::setOriginalLines_(QByteArray const &ba, Git::SubmoduleItem const *submodule, Git::CommitItem const *submodule_commit)
 {
+	(void)submodule;
+	(void)submodule_commit;
+
 	m->original_lines.clear();
+
 	if (!ba.isEmpty()) {
-		char const *begin = ba.data();
-		char const *end = begin + ba.size();
-		misc::splitLines(begin, end, &m->original_lines, true);
+		std::vector<std::string_view> lines = misc::splitLinesV(std::string_view{ba.data(), (size_t)ba.size()}, true);
+		m->original_lines.clear();
+		for (std::string_view const &line : lines) {
+			m->original_lines.push_back(std::string(line));
+		}
 	}
 }
 
-void FileDiffWidget::setLeftOnly(QByteArray const &ba, Git::Diff const &diff)
+void FileDiffWidget::setLeftOnly(Git::Diff const &diff, QByteArray const &ba)
 {
 	m->init_param_ = InitParam_();
 	m->init_param_.view_style = FileDiffWidget::ViewStyle::LeftOnly;
 	m->init_param_.bytes_a = ba;
 	m->init_param_.diff = diff;
 
-	setOriginalLines_(ba);
+	setOriginalLines_(ba, &diff.a_submodule.item, &diff.a_submodule.commit);
 
 	if (setupPreviewWidget() == FileViewType::Text) {
 
@@ -424,16 +486,57 @@ void FileDiffWidget::setLeftOnly(QByteArray const &ba, Git::Diff const &diff)
 	}
 }
 
-void FileDiffWidget::setRightOnly(QByteArray const &ba, Git::Diff const &diff)
+bool FileDiffWidget::setSubmodule(Git::Diff const &diff)
+{
+	Git::SubmoduleItem const &submod_a = diff.a_submodule.item;
+	Git::SubmoduleItem const &submod_b = diff.b_submodule.item;
+	Git::CommitItem const &submod_commit_a = diff.a_submodule.commit;
+	Git::CommitItem const &submod_commit_b = diff.b_submodule.commit;
+	if (submod_a || submod_b) {
+		auto Text = [](Git::SubmoduleItem const *submodule, Git::CommitItem const *submodule_commit, TextDiffLineList *out){
+			*out = {};
+			if (submodule && *submodule) {
+				QString text;
+				text += "name: " + submodule->name + '\n';
+				text += "path: " + submodule->path + '\n';
+				text += "url: " + submodule->url + '\n';
+				text += "commit: " + submodule->id.toQString() + '\n';
+				text += "date: " + misc::makeDateTimeString(submodule_commit->commit_date) + '\n';
+				text += "author: " + submodule_commit->author + '\n';
+				text += "email: " + submodule_commit->email + '\n';
+				text += '\n';
+				text += submodule_commit->message;
+				for (QString const &line : misc::splitLines(text)) {
+					out->push_back(Document::Line(line.toStdString()));
+				}
+			}
+		};
+		TextDiffLineList left_lines;
+		TextDiffLineList right_lines;
+		if (submod_a) {
+			Text(&submod_a, &submod_commit_a, &left_lines);
+		}
+		if (submod_b) {
+			Text(&submod_b, &submod_commit_b, &right_lines);
+		}
+		setDiffText(diff, left_lines, right_lines);
+		return true;
+	}
+	return false;
+}
+
+void FileDiffWidget::setRightOnly(Git::Diff const &diff, QByteArray const &ba)
 {
 	m->init_param_ = InitParam_();
 	m->init_param_.view_style = FileDiffWidget::ViewStyle::RightOnly;
 	m->init_param_.bytes_b = ba;
 	m->init_param_.diff = diff;
 
-	setOriginalLines_(ba);
+	setOriginalLines_(ba, &diff.b_submodule.item, &diff.b_submodule.commit);
 
-	if (setupPreviewWidget() == FileViewType::Text) {
+	if (setSubmodule(diff)) {
+		// ok
+	} else if (setupPreviewWidget() == FileViewType::Text) {
 
 		TextDiffLineList left_lines;
 		TextDiffLineList right_lines;
@@ -447,7 +550,7 @@ void FileDiffWidget::setRightOnly(QByteArray const &ba, Git::Diff const &diff)
 	}
 }
 
-void FileDiffWidget::setSideBySide(QByteArray const &ba, Git::Diff const &diff, bool uncommited, QString const &workingdir)
+void FileDiffWidget::setSideBySide(Git::Diff const &diff, QByteArray const &ba, bool uncommited, QString const &workingdir)
 {
 	m->init_param_ = InitParam_();
 	m->init_param_.view_style = FileDiffWidget::ViewStyle::SideBySideText;
@@ -456,28 +559,33 @@ void FileDiffWidget::setSideBySide(QByteArray const &ba, Git::Diff const &diff,
 	m->init_param_.uncommited = uncommited;
 	m->init_param_.workingdir = workingdir;
 
-	setOriginalLines_(ba);
+	setOriginalLines_(ba, {}, {});
 
-	if (setupPreviewWidget() == FileViewType::Text) {
+	if (setSubmodule(diff)) {
+		// ok
+	} else {
+		if (setupPreviewWidget() == FileViewType::Text) {
 
-		TextDiffLineList left_lines;
-		TextDiffLineList right_lines;
+			TextDiffLineList left_lines;
+			TextDiffLineList right_lines;
 
-		makeSideBySideDiffData(diff, m->original_lines, &left_lines, &right_lines);
+			makeSideBySideDiffData(diff, m->original_lines, &left_lines, &right_lines);
 
-		setDiffText(diff, left_lines, right_lines);
+			setDiffText(diff, left_lines, right_lines);
+		}
 	}
 }
 
-void FileDiffWidget::setSideBySide_(QByteArray const &ba_a, QByteArray const &ba_b, QString const &workingdir)
+void FileDiffWidget::setSideBySide_(Git::Diff const &diff, QByteArray const &ba_a, QByteArray const &ba_b, QString const &workingdir)
 {
 	m->init_param_ = InitParam_();
 	m->init_param_.view_style = FileDiffWidget::ViewStyle::SideBySideImage;
 	m->init_param_.bytes_a = ba_a;
 	m->init_param_.bytes_b = ba_b;
+	m->init_param_.diff = diff;
 	m->init_param_.workingdir = workingdir;
 
-	setOriginalLines_(ba_a);
+	setOriginalLines_(ba_a, {}, {});
 
 	if (setupPreviewWidget() == FileViewType::Text) {
 
@@ -490,11 +598,12 @@ void FileDiffWidget::setSideBySide_(QByteArray const &ba_a, QByteArray const &ba
 	}
 }
 
-QString FileDiffWidget::diffObjects(GitPtr const &g, QString const &a_id, QString const &b_id)
+QString FileDiffWidget::diffObjects(QString const &a_id, QString const &b_id)
 {
 	if (m->text_codec) {
-		Git::Object obj_a = mainwindow()->cat_file_(g, a_id);
-		Git::Object obj_b = mainwindow()->cat_file_(g, b_id);
+		GitRunner g = git();
+		Git::Object obj_a = mainwindow()->catFile(g, a_id);
+		Git::Object obj_b = mainwindow()->catFile(g, b_id);
 		if (obj_b.type == Git::Object::Type::UNKNOWN) {
 			obj_b.type = Git::Object::Type::BLOB;
 		}
@@ -508,16 +617,21 @@ QString FileDiffWidget::diffObjects(GitPtr const &g, QString const &a_id, QStrin
 				file_b.write(m->text_codec->toUnicode(obj_b.content).toUtf8());
 				file_a.close();
 				file_b.close();
-				QString s = g->diff_file(path_a, path_b);
+				QString s = git().diff_file(path_a, path_b);
 				file_a.remove();
 				file_b.remove();
 				return s;
 			}
 		}
 	}
-	return GitDiff::diffObjects(g, a_id, b_id);
+	return GitDiff::diffObjects(git(), a_id, b_id);
 }
 
+/**
+ * @brief コミットIDの検証
+ * @param id
+ * @return
+ */
 bool FileDiffWidget::isValidID_(QString const &id)
 {
 	if (id.startsWith(PATH_PREFIX)) {
@@ -526,63 +640,69 @@ bool FileDiffWidget::isValidID_(QString const &id)
 	return Git::isValidID(id);
 }
 
+/**
+ * @brief 差分ビューを更新
+ * @param info
+ * @param uncommited
+ */
 void FileDiffWidget::updateDiffView(Git::Diff const &info, bool uncommited)
 {
-	GitPtr g = git();
-	if (!g) return;
-	if (!g->isValidWorkingCopy()) return;
+	ASSERT_MAIN_THREAD();
+	
+	GitRunner g = git();
+	if (!g.isValidWorkingCopy()) return;
 
-	if (isValidID_(info.blob.a_id) && isValidID_(info.blob.b_id)) {
-		Git::Object obj_a = cat_file(g, info.blob.a_id);
-		Git::Object obj_b = cat_file(g, info.blob.b_id);
-		QString mime_a = mainwindow()->determinFileType(obj_a.content, true);
-		QString mime_b = mainwindow()->determinFileType(obj_b.content, true);
+	if (isValidID_(info.blob.a_id_or_path) && isValidID_(info.blob.b_id_or_path)) {
+		Git::Object obj_a = catFile(g, info.blob.a_id_or_path);
+		Git::Object obj_b = catFile(g, info.blob.b_id_or_path);
+		std::string mime_a = mainwindow()->determineFileType(obj_a.content);
+		std::string mime_b = mainwindow()->determineFileType(obj_b.content);
 		if (misc::isImage(mime_a) && misc::isImage(mime_b)) {
-			setSideBySide_(obj_a.content, obj_b.content, g->workingRepositoryDir());
+			setSideBySide_(info, obj_a.content, obj_b.content, g.workingDir());
 			return;
 		}
 	}
 
 	{
 		Git::Diff diff;
-		if (isValidID_(info.blob.a_id) && isValidID_(info.blob.b_id)) {
-			std::string text = diffObjects(g, info.blob.a_id, info.blob.b_id).toStdString();
-			GitDiff::parseDiff(text, &info, &diff);
+		if (isValidID_(info.blob.a_id_or_path) && isValidID_(info.blob.b_id_or_path)) {
+			std::string text = diffObjects(info.blob.a_id_or_path, info.blob.b_id_or_path).toStdString();
+			diff = GitDiff::parseDiff(text, &info);
 		} else {
 			diff = info;
 		}
 
 		Git::Object obj;
-		if (isValidID_(diff.blob.a_id)) { // 左が有効
-			obj = cat_file(g, diff.blob.a_id);
-			if (isValidID_(diff.blob.b_id)) { // 右が有効
-				setSideBySide(obj.content, diff, uncommited, g->workingRepositoryDir()); // 通常のdiff表示
+		if (isValidID_(diff.blob.a_id_or_path)) { // 左が有効
+			obj = catFile(g, diff.blob.a_id_or_path);
+			if (isValidID_(diff.blob.b_id_or_path)) { // 右が有効
+				setSideBySide(diff, obj.content, uncommited, g.workingDir()); // 通常のdiff表示
 			} else {
-				setLeftOnly(obj.content, diff); // 右が無効の時は、削除されたファイル
+				setLeftOnly(diff, obj.content); // 右が無効の時は、削除されたファイル
 			}
-		} else if (isValidID_(diff.blob.b_id)) { // 左が無効で右が有効の時は、追加されたファイル
-			obj = cat_file(g, diff.blob.b_id);
-			setRightOnly(obj.content, diff);
+		} else if (isValidID_(diff.blob.b_id_or_path)) { // 左が無効で右が有効の時は、追加されたファイル
+			obj = catFile(g, diff.blob.b_id_or_path);
+			setRightOnly(diff, obj.content);
 		}
 	}
 }
 
-void FileDiffWidget::updateDiffView(QString const &id_left, QString const &id_right, QString const &path)
+/**
+ * @brief 差分ビューを更新
+ * @param id_left
+ * @param id_right
+ * @param path
+ */
+void FileDiffWidget::updateDiffView_(QString const &id_left, QString const &id_right, QString const &path)
 {
-	GitPtr g = git();
-	if (!g) return;
-	if (!g->isValidWorkingCopy()) return;
+	GitRunner g = git();
+	if (!g.isValidWorkingCopy()) return;
 
-	Git::Diff diff;
-	diff.path = path;
-	diff.blob.a_id = id_left;
-	diff.blob.b_id = id_right;
-	diff.mode = "0";
-	std::string text = diffObjects(g, diff.blob.a_id, diff.blob.b_id).toStdString();
-	GitDiff::parseDiff(text, &diff, &diff);
+	std::string text = diffObjects(id_left, id_right).toStdString();
+	Git::Diff diff = GitDiff::parseDiff(text, &diff);
 
-	Git::Object obj = cat_file(g, diff.blob.a_id);
-	setSideBySide(obj.content, diff, false, g->workingRepositoryDir());
+	Git::Object obj = catFile(g, diff.blob.a_id_or_path);
+	setSideBySide(diff, obj.content, false, g.workingDir());
 
 	ui->widget_diff_slider->clear(false);
 
@@ -616,14 +736,20 @@ void FileDiffWidget::scrollTo(int value)
 	ui->verticalScrollBar->setValue(value);
 }
 
+/**
+ * @brief 縦スクロールバーが操作された
+ */
 void FileDiffWidget::onVerticalScrollValueChanged(int)
 {
-	refrectScrollBar();
+	refrectScrollBarV();
 }
 
+/**
+ * @brief 横スクロールバーが操作された
+ */
 void FileDiffWidget::onHorizontalScrollValueChanged(int)
 {
-	refrectScrollBar();
+	refrectScrollBarH();
 }
 
 void FileDiffWidget::onDiffWidgetWheelScroll(int lines)
@@ -650,19 +776,18 @@ void FileDiffWidget::onDiffWidgetResized()
 
 void FileDiffWidget::on_toolButton_fullscreen_clicked()
 {
-	if (m->init_param_.diff.blob.a_id.isEmpty() && m->init_param_.diff.blob.b_id.isEmpty()) {
+	if (m->init_param_.diff.blob.a_id_or_path.isEmpty() && m->init_param_.diff.blob.b_id_or_path.isEmpty()) {
 		return;
 	}
 
 	BigDiffWindow win(mainwindow());
 	win.setWindowState(Qt::WindowMaximized);
-	win.init(mainwindow(), m->init_param_);
+	win.init(m->init_param_);
 	win.exec();
 }
 
 void FileDiffWidget::setFocusAcceptable(Qt::FocusPolicy focuspolicy)
 {
-//	Qt::FocusPolicy focuspolicy = f ? Qt::ClickFocus : Qt::NoFocus;
 	ui->widget_diff_left->setFocusPolicy(focuspolicy);
 	ui->widget_diff_right->setFocusPolicy(focuspolicy);
 }
@@ -675,13 +800,80 @@ void FileDiffWidget::onUpdateSliderBar()
 	ui->widget_diff_slider->setScrollPos(total, value, page);
 }
 
-void FileDiffWidget::refrectScrollBar()
+/**
+ * @brief スクロールバーの状態を反映
+ * @param updateformat
+ */
+void FileDiffWidget::refrectScrollBar(/*bool updateformat*/)
 {
 	ui->widget_diff_left->refrectScrollBar();
 	ui->widget_diff_right->refrectScrollBar();
+
+	const bool updateformat = true;
+	if (updateformat) {
+
+		// 左と右のテキストを取得
+		TextEditorView::FormattedLines *llines = ui->widget_diff_left->getTextEditorView()->fetchLines();
+		TextEditorView::FormattedLines *rlines = ui->widget_diff_right->getTextEditorView()->fetchLines();
+
+		// サイドバイサイドのとき文字差分を求める
+		if (viewstyle() == SideBySideText) {
+			using Char = AbstractCharacterBasedApplication::Char;
+			using Attr = AbstractCharacterBasedApplication::CharAttr;
+
+			const size_t n = std::min(llines->size(), rlines->size());
+
+			for (std::size_t i = 0; i < n; i++) {
+				std::vector<Char> *lchars = llines->chars(i);
+				std::vector<Char> *rchars = rlines->chars(i);
+				size_t l = 0;
+				size_t r = 0;
+
+				// 文字差分を求める
+				// dtl (diff template library) https://github.com/cubicdaiya/dtl
+				dtl::Diff<uint32_t, std::vector<Char>> d(*lchars, *rchars);
+				d.compose();
+
+				auto const &sq = d.getSes().getSequence();
+				for (std::pair<uint32_t, dtl::elemInfo> const &t : sq) {
+					switch (t.second.type) {
+					case dtl::SES_COMMON:
+						l++;
+						r++;
+						break;
+					case dtl::SES_DELETE:
+						lchars->at(l).attr.flags |= Attr::Underline1;
+						l++;
+						break;
+					case dtl::SES_ADD:
+						rchars->at(r).attr.flags |= Attr::Underline2;
+						r++;
+						break;
+					}
+				}
+			}
+		}
+	}
+
 	onUpdateSliderBar();
 }
 
+/**
+ * @brief 縦スクロールバーの状態を反映
+ */
+void FileDiffWidget::refrectScrollBarV()
+{
+	refrectScrollBar();
+}
+
+/**
+ * @brief 横スクロールバーの状態を反映
+ */
+void FileDiffWidget::refrectScrollBarH()
+{
+	refrectScrollBar();
+}
+
 QPixmap FileDiffWidget::makeDiffPixmap(DiffPane pane, int width, int height)
 {
 	auto Do = [&](TextDiffLineList const &lines){
@@ -692,6 +884,13 @@ QPixmap FileDiffWidget::makeDiffPixmap(DiffPane pane, int width, int height)
 	return QPixmap();
 }
 
+/**
+ * @brief スクロール位置の行と桁を設定
+ * @param cur_row
+ * @param cur_col
+ * @param scr_row
+ * @param scr_col
+ */
 void FileDiffWidget::onMoved(int cur_row, int cur_col, int scr_row, int scr_col)
 {
 	(void)cur_col;
@@ -702,7 +901,7 @@ void FileDiffWidget::onMoved(int cur_row, int cur_col, int scr_row, int scr_col)
 	onUpdateSliderBar();
 }
 
-void FileDiffWidget::setTextCodec(QTextCodec *codec)
+void FileDiffWidget::setTextCodec(std::shared_ptr<MyTextCodec> codec)
 {
 	m->text_codec = codec;
 	ui->widget_diff_left->setTextCodec(codec);
@@ -712,12 +911,14 @@ void FileDiffWidget::setTextCodec(QTextCodec *codec)
 
 void FileDiffWidget::setTextCodec(char const *name)
 {
-	QTextCodec *codec = name ? QTextCodec::codecForName(name) : nullptr;
+	std::shared_ptr<MyTextCodec> codec = std::make_shared<MyTextCodec>(name);
+	// MyTextCodec *codec = name ? MyTextCodec::codecForName(name) : nullptr;
 	setTextCodec(codec);
 }
 
 void FileDiffWidget::on_toolButton_menu_clicked()
 {
+#if 0
 	QMenu menu;
 	QAction *a_utf8 = menu.addAction("UTF-8");
 	QAction *a_sjis = menu.addAction("SJIS (CP932)");
@@ -742,4 +943,5 @@ void FileDiffWidget::on_toolButton_menu_clicked()
 			return;
 		}
 	}
+#endif
 }
