@@ -1,23 +1,25 @@
 #ifndef FILEPROPERTYDIALOG_H
 #define FILEPROPERTYDIALOG_H
 
+#include "Git.h"
+
 #include <QDialog>
 
 namespace Ui {
 class FilePropertyDialog;
 }
 
-class BasicMainWindow;
+class MainWindow;
 
 class FilePropertyDialog : public QDialog {
 	Q_OBJECT
 private:
-	BasicMainWindow *mainwindow;
+	MainWindow *mainwindow;
 public:
 	explicit FilePropertyDialog(QWidget *parent = nullptr);
 	~FilePropertyDialog() override;
 
-	void exec(BasicMainWindow *mw, QString const &path, QString const &id);
+	void exec(QString const &path, const Git::Hash &id);
 private:
 	Ui::FilePropertyDialog *ui;
 };
