@@ -1,6 +1,8 @@
 #ifndef MYPROCESS_H
 #define MYPROCESS_H
 
+// Process.h というファイルはWindows環境に既存なので MyProcess.h にした
+
 #include <QtGlobal>
 
 #ifdef Q_OS_WIN
@@ -15,10 +17,13 @@ using Process = UnixProcess;
 using PtyProcess = UnixPtyProcess;
 #endif
 
-class misc2 {
+class ProcessStatus {
 public:
-	static int runCommand(QString const &cmd, QByteArray *out);
-	static int runCommand(QString const &cmd, QByteArray const *in, QByteArray *out);
+	bool ok = false;
+	int exit_code = 0;
+	std::string error_message;
+	std::string log_message;
 };
+Q_DECLARE_METATYPE(ProcessStatus)
 
 #endif // MYPROCESS_H
