@@ -1,7 +1,7 @@
 #include "MySettings.h"
 #include "ApplicationGlobal.h"
 
-MySettings::MySettings(QObject *)
+MySettings::MySettings()
 	: QSettings(global->config_file_path, QSettings::IniFormat)
 {
 }
