@@ -1,22 +1,37 @@
 #include "MyTextEditorWidget.h"
+#include "ApplicationGlobal.h"
 #include "MainWindow.h"
-#include "common/misc.h"
-#include <QMenu>
-#include <QFileDialog>
 #include "common/joinpath.h"
+#include "common/misc.h"
+#include <QFileDialog>
+#include <QFontDatabase>
+#include <QMenu>
 
 MyTextEditorWidget::MyTextEditorWidget(QWidget *parent)
-	: TextEditorWidget(parent)
+	: TextEditorView(parent)
 {
-
+	QFont font = QFontDatabase::systemFont(QFontDatabase::FixedFont);
+	view()->setTextFont(font);
 }
 
-void MyTextEditorWidget::setDocument(const QList<Document::Line> *source, BasicMainWindow *mw, QString const &object_id, QString const &object_path)
+TextEditorView *MyTextEditorWidget::view()
+{
+	return this;
+}
+
+void MyTextEditorWidget::clear()
+{
+	this->object_id.clear();
+	this->object_path.clear();
+	view()->clear();
+	update();
+}
+
+void MyTextEditorWidget::setDocument(const QList<Document::Line> *source, QString const &object_id, QString const &object_path)
 {
-	this->mainwindow = mw;
 	this->object_id = object_id;
 	this->object_path = object_path;
-	TextEditorWidget::setDocument(source);
+	view()->setDocument(source);
 }
 
 void MyTextEditorWidget::contextMenuEvent(QContextMenuEvent *event)
@@ -39,19 +54,18 @@ void MyTextEditorWidget::contextMenuEvent(QContextMenuEvent *event)
 		QAction *a = menu.exec(misc::contextMenuPos(this, event));
 		if (a) {
 			if (a == a_save_as) {
-				QString path = mainwindow->currentWorkingCopyDir() / object_path;
+				QString path = global->mainwindow->currentWorkingCopyDir() / object_path;
 				QString dstpath = QFileDialog::getSaveFileName(window(), tr("Save as"), path);
 				if (!dstpath.isEmpty()) {
-					mainwindow->saveAs(id, dstpath);
+					global->mainwindow->saveAs(id, dstpath);
 				}
 				update();
 				return;
 			}
 			if (a == a_copy) {
-				editCopy();
+				view()->editCopy();
 				return;
 			}
 		}
 	}
-
 }
