@@ -3,18 +3,19 @@
 
 #include "texteditor/TextEditorWidget.h"
 
-class BasicMainWindow;
+class MainWindow;
 
-class MyTextEditorWidget : public TextEditorWidget {
+class MyTextEditorWidget : public TextEditorView {
 private:
-	BasicMainWindow *mainwindow;
 	QString object_id;
 	QString object_path;
-public:
-	MyTextEditorWidget(QWidget *parent = nullptr);
-	void setDocument(const QList<Document::Line> *source, BasicMainWindow *mw, QString const &object_id, QString const &object_path);
 protected:
 	void contextMenuEvent(QContextMenuEvent *event) override;
+public:
+	MyTextEditorWidget(QWidget *parent = nullptr);
+	void setDocument(const QList<Document::Line> *source, QString const &object_id, QString const &object_path);
+	TextEditorView *view();
+	void clear();
 };
 
 #endif // MYTEXTEDITORWIDGET_H
