@@ -3,6 +3,7 @@
 
 #include "MyTableWidgetDelegate.h"
 #include "GitHubAPI.h"
+#include "RepositorySearchResultItem.h"
 
 #include <QDialog>
 
@@ -11,20 +12,20 @@ class SearchFromGitHubDialog;
 }
 
 class QTableWidgetItem;
-class BasicMainWindow;
+class MainWindow;
 
 class SearchFromGitHubDialog : public QDialog {
 	Q_OBJECT
 private:
 	Ui::SearchFromGitHubDialog *ui;
-	QList<GitHubAPI::SearchResultItem> items;
+	QList<RepositorySearchResultItem> items;
 	QString url_;
 	MyTableWidgetDelegate item_delegate;
-	BasicMainWindow *mainwindow;
+	MainWindow *mainwindow;
 
 	void updateUI();
 public:
-	explicit SearchFromGitHubDialog(QWidget *parent, BasicMainWindow *mw);
+	explicit SearchFromGitHubDialog(QWidget *parent, MainWindow *mw);
 	~SearchFromGitHubDialog() override;
 
 	QString url() const;
