@@ -10,8 +10,7 @@ namespace Ui {
 class SettingGeneralForm;
 }
 
-class SettingGeneralForm : public AbstractSettingForm
-{
+class SettingGeneralForm : public AbstractSettingForm {
 	Q_OBJECT
 private:
 	Ui::SettingGeneralForm *ui;
@@ -28,7 +27,6 @@ public:
 	static QList<Languages::Item> languages();
 	static void execSelectLanguageDialog(QWidget *parent, const QList<Languages::Item> &langs, const std::function<void ()> &done);
 private slots:
-	void on_pushButton_browse_default_working_dir_clicked();
 	void on_pushButton_change_language_clicked();
 	void on_pushButton_change_theme_clicked();
 };
